/*
    Copyright (C) 2009 José Millán Soto <fid@gpul.org>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "contactnotifier.h"
#include "ui_contactnotifier.h"

#include <kopetemetacontact.h>
#include <kopetecontactlist.h>
#include <kopetechatsessionmanager.h>
#include <kopetemetacontact.h>
#include <kopetechatsession.h>
#include <kopetemessage.h>
#include <kopeteaccount.h>
#include <kopetecontact.h>

#include <ksharedconfig.h>
#include <kgenericfactory.h>
#include <kactioncollection.h>
#include <kaction.h>
#include <QDialog>
#include <QList>


K_PLUGIN_FACTORY(ContactNotifierPluginFactory, registerPlugin<ContactNotifier>();)
K_EXPORT_PLUGIN(ContactNotifierPluginFactory( "kopete_contactnotifier" ))

ContactNotifier::ContactNotifier( QObject *parent, const QVariantList & /* args */ )
    : Kopete::Plugin(ContactNotifierPluginFactory::componentData(), parent )
{
    KAction *addDelayedMessageAction = actionCollection()->addAction( "contactNotificationDialog" );
    addDelayedMessageAction->setText( i18n("Set contact notification message") );
    setXMLFile( "contactnotifier.rc" );
    connect( addDelayedMessageAction, SIGNAL( triggered() ), this,
      SLOT( slotShowMessageDialog() ) );
    connect ( this , SIGNAL( settingsChanged() ) , this ,
      SLOT( loadConfig() ) );
    loadConfig();
}

void ContactNotifier::sendMessage(Kopete::Contact *c, QString messageText) {
    Kopete::ChatSession *s = c->manager(Kopete::Contact::CanCreate);
    Kopete::Message msg(c->account()->myself(), c);
    msg.setPlainBody(messageText);
    msg.setDirection(Kopete::Message::Outbound);
    s->sendMessage(msg);
}

void ContactNotifier::slotOnlineStatusChanged(Kopete::MetaContact *contact,
        Kopete::OnlineStatus::StatusType status) {
    if ( (status == Kopete::OnlineStatus::Online) ||
       ( (status == Kopete::OnlineStatus::Away) && m_sendIfAway ) ){
      QString k = contact->metaContactId().toString();
      if ( !m_messages.contains(k) ) return;
      QString messageText = m_messages[k];
      if ( messageText.size() == 0 ) return;
      Kopete::Contact *pref = contact->preferredContact();
      if (pref) {
          sendMessage(pref, messageText);
          m_messages.remove(k);
      }
    }
}

void ContactNotifier::slotShowMessageDialog() {
    foreach(Kopete::MetaContact *m, Kopete::ContactList::self()->selectedMetaContacts()) {
      QDialog *d = new QDialog();
      Ui::ContactNotificationDialog ui;
      ui.setupUi(d);
      QLabel *l = d->findChild<QLabel *>( "m_label" );
      QTextEdit *t = d->findChild<QTextEdit *>( "m_delayedMessageEdit" );
      if ( !l || !t ) {
        delete d;
        return;
      }
      l->setText( i18n("Message for: ") + m->displayName() );
      QString k = m->metaContactId().toString();
      if ( m_messages.contains( k ) ) {
        t->setText( m_messages[k] );
      }
      m_dialogs[d] = k;
      connect( d, SIGNAL(accepted()), this, SLOT(slotAcceptDialog()) );
      connect( d, SIGNAL(rejected()), this, SLOT(slotRejectDialog()) );
      connect( m, SIGNAL( onlineStatusChanged (Kopete::MetaContact *, Kopete::OnlineStatus::StatusType) ),
          this, SLOT(slotOnlineStatusChanged(Kopete::MetaContact *, Kopete::OnlineStatus::StatusType) ) );
      d->exec();
    }
}

void ContactNotifier::slotAcceptDialog() {
    QDialog *d = dynamic_cast<QDialog *>( sender() );
    if (d) {
        QString u = m_dialogs[d];
        QTextEdit *t = d->findChild<QTextEdit *>( "m_delayedMessageEdit" );
        if (t) {
            m_messages[u] = t->toPlainText();
        }
        m_dialogs.remove(d);
        delete d;
    }
}

void ContactNotifier::slotRejectDialog() {
    QDialog *d = dynamic_cast<QDialog *>( sender() );
    if (d) {
      m_dialogs.remove(d);
      delete d;
    }
}

void ContactNotifier::loadConfig() {
    KSharedConfigPtr cPtr = KSharedConfig::openConfig();
    KConfigGroup cGroup(cPtr, "ContactNotifier");
    m_sendIfAway = cGroup.readEntry<bool>("SendIfAway", false);
}
